#include "QPdef.h"

/* global variables */
FILE *fmps=NULL,
     *fres=NULL;
char *nmps=NULL,
     *nspc=NULL;


qpdat *coplqp_qt =NULL;
// qppar *pa =NULL;
// TODO

clock_t t_start;
clock_t t_end;

clock_t GetTime(void)
{
  clock_t t;

#ifdef UNIXMACHINE  
  struct tms {
    clock_t    tms_utime;
    clock_t    tms_stime;
    clock_t    tms_cutime;
    clock_t    tms_cstime;
  } tmrec;

  //clock_t times();

  t = times(&tmrec);
  t = tmrec.tms_utime+tmrec.tms_stime;
#else
  t = clock();
#endif

  return t;
} /* GetTime */

static void PrintHead(void)
{
  char Title[8][60]={
    " ==========================================\n",
    " Computational Optimization Program Library\n",
    "                                           \n",
    "     (  C  O  P  L  )     Version 1.0      \n",
    "                                           \n",
    " Computational Optimization Lab.           \n",
    " The University of Iowa (1997)             \n",
    " ==========================================\n"};
  int i;

  printf("\n\n\n");
  for (i=0; i<8; i++)
    printf("%s", Title[i]);
  printf("\n ****************************** COPL STARTS");
  printf(" *******************************\n\n");

  return;
} /* PrintHead */

static int FprintHead(void)
{
  printf("====================\n");
  printf("C O P L  1.0  (1997)\n");
  printf("====================\n\n");
  return true;
} /* FprintHead */

static int CheckArgs(int   argc,
                     char* argv[],
                     char* ss)
{
  int i,id;
  
  id = 0;
  for (i=1; i<argc; i++) {
    if(strstr(argv[i],ss)) {
      argv[i]+=2;
      if (argv[i][0] == ':') {
        argv[i]++;
        id=i;
        break;
      }
      if (argv[i][0]) {
        id=i;
        break;
      }
      id = i+1;
      if (argv[id][0] == '-')
        id = 0;
      break;
    }
  }
  return id;
} /* Checkargs */

void setParams(qppar* par)
{
/*
  char   parname[][50]=
  {
    "bound_set_name",
    "cache_memory_size",
    "dual_feasible_tolerance",
    "initial_point_threshold",
    "iteration_limit",
    "lower_bound_default",
    "number_of_columns",
    "objective_set_name",
    "primal_dual_gap_tolerance",
    "primal_feasible_tolerance",
    "problem_type",
    "range_set_name",
    "rignt_hand_side_set_name",
    "step_factor",
    "upper_bound_default",
    ""
  };
*/

  par->q_intCol=       100000;
  par->q_intItr=          200;
  par->q_intCac=          256;
  par->q_intMin=         true;
    
  par->q_strRhs=         NULL;
  par->q_strRng=         NULL;
  par->q_strBnd=         NULL;
  par->q_strObj=         NULL;
    
  par->q_dblLow=          0.0;
  par->q_dblUpp=        1e+40;
  par->q_dblMax=0.999*_P_MAXM;
  par->q_dblAij=        1e-10;    
  par->q_dblPiv=        1e-18;
  par->q_dblGap=        1e-06;
  par->q_dblPri=        1e-06;
  par->q_dblDua=        1e-06;
  par->q_dblStp=         0.95;
  par->q_dblRad=        1e-08;
  par->q_dblIni=        1000.;
} /* setParams */

static void GetFile(char fname[])
{
  int  k;
  char *ptr;

  printf(" Please type the name of your data file: ");
  fflush(stdin);
  fgets(fname,128,stdin);
  k=strlen(fname);
  ptr=fname+k-1;
  while(k>0&&(*ptr=='\n'||*ptr==' '))
  {
    *ptr='\0';
    ptr--;
    k--;
  }
} /* GetFile */

int SolveQP(qpdat *qpdata)
{
  int   idKey; 
  qpdat qp_backup, *qp_bak;
  qppar *p = &qpdata->par;

  t_start=GetTime();

  //setParams(p);
  //
  coplqp_qt = qpdata;

  qp_bak = &qp_backup; 
  memcpy(qp_bak, qpdata, sizeof(qpdat));

  idKey=PreProc(p,qpdata, qp_bak);
  if (OPT_OK!=idKey)
    return idKey;

  /* symbolic computation */
  idKey=SymboProc(qpdata,p);
  if (OPT_OK!=idKey)
    return idKey;
  
  /* numerical computation */
  idKey=QpSolv(qpdata,p, qp_bak);
  
//  ShutDown(qpdata->m_intSta);

  return idKey;
} 


/*
int main(int argc,char* argv[])
{  
  // local variables 
  int   j, idKey; 
  qpdat qp={0};
  char *nout;
  
  /// initialization 
  PrintHead();

  idKey=CheckArgs(argc,argv,"-o");
  if (!idKey) {
    nout=cAlloc(20);
    sprintf(nout,"coplfile.out");
  }
  else nout=argv[idKey];

  if (!FprintHead())
    return NOT_DSKSPC;
  
  idKey=CheckArgs(argc,argv,"-s");
  if (idKey) nspc=argv[idKey];

  setParams(&qp.par);
  
  // input data 
  idKey=CheckArgs(argc,argv,"-f");
  if (!idKey) {
    nmps =cAlloc(128);
    GetFile(nmps);
  }
  else nmps=argv[idKey];

  idKey=DataInput(&qp.par,&qp,nmps);
  if (OPT_OK!=idKey)
    return idKey;
 //USE 
  SolveQP(&qp);
  printf(" Primal varibles: 		Dual:");

  for(j=0;j<qp.m_colUse;j++) 
	  printf("         x[%2i]=%g		z[%2i]=%g\n", j+1, qp.x[j], j+1, qp.rmem[j]);

  // USE
  ShutDown(qp.m_intSta);
  return 0;
}  

*/

/* main */
