
#include "cQpGenDense.h"
#include "solver.h"

#define PRINT_LEVEL 0

typedef struct _qp_ooqp_program_ {
// these variables are used to store constraint data before the constraint type is known
	int exp_row;
	double *curA;

	SolverConstraintType *a_type;

// min == 1 if the objective function should be minimized, otherwise ==0
	char min; 

// total number of constraints 
	int myz;
/**
 * The general quadratic formulation recognized by OOQP is as follows:
 *				min 1/2 x'Qx + c' x subject to
 *					Ax = b, 
 *					d <= Cx <= f,
 *					l <= x <= u
 *
 **/

// nx is the number of primal variables. It is the length of the input vectors
// c, xlow, ixlow, xupp, ixupp, x, gamma, and phi.
	int nx;

// c is the linear term in the objective function, a vector of length nx
// Q is objective function matrix
	double    *c, *Q;

//xlow, ixlow are the lower bounds on x. These contain the information in the
//lower bounding vector l in the formulation given above. 
//If there is a bound on element k of x (that is, lk > -1), then xlow[k] should
//be set to the value of lk and ixlow[k] should be set to one. 
//Otherwise, element k of both arrays should be set to zero.
double  *xlow;
char   	*ixlow;

//xupp, ixupp are the upper bounds on x, that is, the information in the vector
//u in the formulation given above. These should be defined in a similar fashion to 
//xlow and ixlow.
double  *xupp;
char   *ixupp;

// bA contains the right-hand-side vector b for the equality constraints.
// The integer parameter my defines the length of this vector. 
// A contains the coefficients of x for the linear equality constraints.
	int my;
	double *A;
	double *bA;

// The integer parameter mz defines the number of inequality constraints.
	int mz;

// clow, iclow are the lower bounds of the inequality constraints.
	double *clow;
	char  *iclow;

// cupp, icupp are the upper bounds of the inequality constraints.
	 double *cupp;
	 char  *icupp;

// C contains the coefficients of x for the inequality constraints.
	 double   *C;

	 double *x, *gamma, *phi, ObjVal;

  /* double y[my];
 *    * y contains the multipliers for the equality constraints Ax = b.
 *       */
	 double *y;

  /* double z[mz], lambda[mz], pi[mz];
 *    * lambda and pi contain the multipliers for the inequality constraints
 *       * Cx >= d and Cx <= f, respectively.
 *          */
	 double *z, *lambda, *pi;

// cupp, icupp are the upper bounds of the inequality constraints. 
} qp_ooqp_program;
